//
//    Title: imagergba.h
//  Created: Tue Oct  2 18:27:26 2001
//   Author: Tim Weyrich <weyrich@inf.ethz.ch>
//
// copyright (c) 2001--2003, Computer Graphics Laboratory, ETH Zuerich
//

#ifndef __IMAGERGBA_H__
#define __IMAGERGBA_H__

//! C-struct that describes an rgba pixel-value.
typedef struct
{
  unsigned char  r;             //!< red value
  unsigned char  g;             //!< green value
  unsigned char  b;             //!< blue value
  unsigned char  a;             //!< alpha value
} Rgba;

//! C-struct that holds an rgba image.
/*! Use imageRgbaNew() and imageRgbaFree() to create or to delete an
 *  ImageRGBA struct, respectively.
 */
typedef struct
{
  int    width;                 //!< Width of the image.
  int    height;                //!< Height of the image.
  
  int    bytesPerPixel;         //!< Number of bytes per Pixel (always 4).
  
  //! Pixel data of the image.
  /*! The data is organized in row-major order. That is, pixels of an
   *  image row are stored subsequently from left to right, and rows
   *  are stored subsequently from top to bottom.
   */
  Rgba  *data;
} ImageRGBA;


//! Creates a new rgba image.
extern ImageRGBA  *imageRgbaNew(int width, int height);

//! Deletes an rgba image.
extern void        imageRgbaFree(ImageRGBA *img);


//! Creates a copy of an rgba image.
extern ImageRGBA  *imageRgbaClone(const ImageRGBA *img);


//! Clears an rgba image.
extern void        imageRgbaClear(ImageRGBA *img);

//! Sets all pixels in the image to a given value.
extern void        imageRgbaSet(ImageRGBA *img,
                                unsigned r, unsigned g, unsigned b, unsigned a);

//! Reads an rgba image from a TIFF file.
extern ImageRGBA  *imageRgbaReadTIFF(const char *fname);

//! Writes an rgba image as a TIFF file.
extern int         imageRgbaWriteTIFF(const char *fname, const ImageRGBA *img);


//! Draws a line into a color image.
/*! The line is drawn with pixels of color \a colorValue from the
 *  point (\a x0, \a y0) to (\a x1, \a y1).
 */
extern void  imageRgbaDrawLine(ImageRGBA *img, int x0, int y0, int x1, int y1, Rgba colorValue);

//! Draws a line into a color image by inverting pixels.
/*! The line is drawn from the point (\a x0, \a y0) to (\a x1, \a y1).
 */
extern void  imageRgbaInvertLine(ImageRGBA *img, int x0, int y0, int x1, int y1);

#endif
